%
% References :
%   -
%
% Author:  Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib
% Date:    2021-05-29
% Version: 1.0
% Umail:   jordan.caracotte@etud.u-picardie.fr
% URL:     https://mis.u-picardie.fr/~fabio/PhotoSphere.html
%
% (c) 2021 Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib, 
% MIS Laboratory, University of Picardie Jules Verne, Amiens, France
%
% History:
%   Date:       Version       Comment:
%   2019-06-08  1.0.20190608  Original Version
%   2020-04-15  1.0.20200415  Licence update
%   2021-05-29  1.0.20210529  Add missing file

function [N, p, q] = compute_normals_single_fisheye(images, mask, lights, camera_parameters)
    [m, n] = size(mask);

    % Compute 3D-point coordinates on the unit sphere
    [xs, ys, zs, ~, ~] = reverseProjection(m, n, camera_parameters);

    % Filter
    xs = real(xs) .* mask;
    ys = real(ys) .* mask;
    zs = real(zs) .* mask;

    % Compute phi & theta
    theta = real(acos(zs));
    cosPhi = xs./sin(theta);
    cosPhi(isnan(cosPhi)) = 1;
    sinPhi = ys./sin(theta);
    sinPhi(isnan(sinPhi)) = 0;
    phi = atan2(real(sinPhi), real(cosPhi));

    p = zeros(m*n, 1);
    q = zeros(m*n, 1);
    N = zeros(m*n, 3);

    mask = mask(:);
    theta = theta(:);
    phi = phi(:);

    parfor i = 1:m*n
        if mask(i) ~= 0 && theta(i) ~= 0
            [ p_, q_ ] = computePQ(images(i, :), lights, theta(i), phi(i));
                
            p(i) = -p_;
            q(i) = -q_;
                
            normal = 1/sqrt(p(i)^2 + (q(i)/sin(theta(i)))^2 + 1) * rotoz(phi(i)) * rotoy(theta(i)) * [ p(i), q(i)/sin(theta(i)), -1 ]';
            N(i, :) = normal;
        end
    end

    N = reshape(N, [m, n, 3]);
end
