%
% References :
%   -
%
% Author:  Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib
% Date:    2020-04-15
% Version: 1.0
% Umail:   jordan.caracotte@etud.u-picardie.fr
% URL:     https://mis.u-picardie.fr/~fabio/PhotoSphere.html
%
% (c) 2020 Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib, 
% MIS Laboratory, University of Picardie Jules Verne, Amiens, France
%
% History:
%   Date:       Version       Comment:
%   2020-04-15  1.0.20200415  Original Version
%

function ln_rho = rangeMapOmni(p, q, lambda)
[rows, cols] = size(p);

%pointCount = rows*cols;
pointCount = rows*cols + 1;
%eqCount = 2 * ((rows-1) * cols) + rows;
eqCount = 2 * ((rows-1) * cols) + rows + 2*cols;

M = sparse(eqCount, pointCount);
b = zeros(eqCount, 1);

eqId = 0;
inverseOfdTheta = lambda/pi; % UnSurDetheta

for row = 1:rows-1
    for col = 1:cols-1
        eqId = eqId + 1;

        M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
        M(eqId, (row-1)*cols + col + 1) = inverseOfdTheta;
        b(eqId, 1) = q(row, col);
        
        eqId = eqId + 1;
        M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
        M(eqId, row*cols + col) = inverseOfdTheta;
        b(eqId, 1) = p(row, col);
    end
    
    % Last column (circularity)
    col = cols;
    eqId = eqId + 1;
    M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
    M(eqId, (row-1)*cols + 1) = inverseOfdTheta;
    b(eqId, 1) = q(row, col);
   
    eqId = eqId + 1;
    M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
    M(eqId, row*cols + col) = inverseOfdTheta;
    b(eqId, 1) = p(row, col);
end

% Last row
row = rows;
for col = 1:cols-1
    eqId = eqId + 1;
    M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
    M(eqId, (row-1)*cols + col + 1) = inverseOfdTheta;
    b(eqId, 1) = q(row, col);
end

% Last column & last row (circularity)
col = cols;
eqId = eqId + 1;
M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
M(eqId, (row-1)*cols + 1) = inverseOfdTheta;
b(eqId, 1) = q(row, col);

% Poles!
% North-Pole
%row = 1;
%for col = 1:cols
%    eqId = eqId + 1;
%    M(eqId, row*cols+col) = -inverseOfdTheta;
%    M(eqId, rows*cols + 1) = inverseOfdTheta;
%    b(eqId, 1) = -p(row, col);
%end

% South-Pole
%row = rows;
%for col = 1:cols
%    eqId = eqId + 1;
%    M(eqId, row*cols+col) = -inverseOfdTheta;
%    M(eqId, rows*cols + 1) = inverseOfdTheta;
%    b(eqId, 1) = p(row, col);
%end

X = M \ b;

ln_rho = zeros(row, col);
eqId = 0;

for row = 1:rows
    for col = 1:cols
        eqId = eqId + 1;
        ln_rho(row, col) = X(eqId, 1);
    end
end
