%
% References :
%   -
%
% Author:  Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib
% Date:    2020-04-15
% Version: 1.0
% Umail:   jordan.caracotte@etud.u-picardie.fr
% URL:     https://mis.u-picardie.fr/~fabio/PhotoSphere.html
%
% (c) 2020 Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib, 
% MIS Laboratory, University of Picardie Jules Verne, Amiens, France
%
% History:
%   Date:       Version       Comment:
%   2020-04-15  1.0.20200415  Original Version
%

clear; close all

%% Camera parameters
camera_parameters0 = {};
camera_parameters0.ku = 2492.753421;
camera_parameters0.u0 = 4310.955178-2896;
camera_parameters0.kv = 2495.964728;
camera_parameters0.v0 = 1464.089267;
camera_parameters0.xi = 1.866312114;

camera_parameters1 = {};
camera_parameters1.ku = 2484.116737;
camera_parameters1.u0 = 1457.187043;
camera_parameters1.kv = 2505.459463;
camera_parameters1.v0 = 1466.278648;
camera_parameters1.xi = 1.847711275;

c1Mc0 = [-0.9996981518,    -0.0005242698754,  0.02456278658, 0 ;
          0.0002307654689,  0.9995278159,     0.03072607963, 0 ;
         -0.02456729718,    0.03072247326,   -0.9992259892,  0 ;
          0                 0                 0              1 ];

%% Light directions
load '../synthetic/geodesic_polyhedron/lights'
lights = lights./sqrt(sum(lights.^2));
lights1 = lights;
lights0 = c1Mc0 * [ lights; zeros(1, 14) ];
lights0 = lights0(1:3, :);
lights0 = lights0./sqrt(sum(lights0.^2));

%% Masks
mask1 = ones(2896, 2896);
mask0 = ones(2896, 2896);

%% Images
imageCount = 14;

images0 = zeros(2896*2896, imageCount);
images1 = zeros(2896*2896, imageCount);

rgb_channel = 1; % r = 1, g = 2, b = 3

path = '';
im_path = '../synthetic/geodesic_polyhedron/png/';

for i = 1:imageCount
    tmpI = double(imread([ path, im_path, sprintf('%d', i) '.png']))/255.0;
    
    % Filter
    %mask = zeros(2896, 2896*2);
    %mask(:, 1:2896) = mask1;
    %mask(:, 2897:end) = mask0;
    %mask = imbinarize(mask);
    %tmpI = tmpI .* mask;
    
    % Normalisation
    %minI = min(tmpI(:));
    %maxI = max(tmpI(:));
    %tmpI = (tmpI - minI)/(maxI - minI);
    
    subimage1 = tmpI(:, 1:2896, rgb_channel);
    subimage0 = tmpI(:, 2897:end, rgb_channel);

    images1(:, i) = subimage1(:);
    images0(:, i) = subimage0(:);
end

%% Normals
[N1, estimated_p1, estimated_q1] = compute_normals_single_fisheye(images1, mask1, lights1, camera_parameters1);
[N0, estimated_p0, estimated_q0] = compute_normals_single_fisheye(images0, mask0, lights0, camera_parameters0);

estimated_p1 = reshape(estimated_p1, [2896, 2896]);
estimated_q1 = reshape(estimated_q1, [2896, 2896]);
estimated_p0 = reshape(estimated_p0, [2896, 2896]);
estimated_q0 = reshape(estimated_q0, [2896, 2896]);

%% Fusion
lambda = 512;

incr = pi/lambda;

theta1 = incr : incr : pi/2 - incr;
phi1 = 0 : incr : 2*pi - incr;
theta0 = pi/2 + incr : incr : pi - incr;
phi0 = 0 : incr : 2*pi - incr;

[p1, t1] = meshgrid(phi1, theta1);
xs1 = sin(t1).*cos(p1);
ys1 = sin(t1).*sin(p1);
zs1 = cos(t1);

[p0, t0] = meshgrid(phi0, theta0);
xs0 = sin(t0).*cos(p0);
ys0 = sin(t0).*sin(p0);
zs0 = cos(t0);

P = [ xs0(:), ys0(:), zs0(:), ones(length(theta0)*length(phi0), 1) ]';
Pp = c1Mc0 * P;
xs0 = reshape(Pp(1, :)./Pp(4, :), [length(theta0), length(phi0)]);
ys0 = reshape(Pp(2, :)./Pp(4, :), [length(theta0), length(phi0)]);
zs0 = reshape(Pp(3, :)./Pp(4, :), [length(theta0), length(phi0)]);

[~, ~, ~, x1, y1] = reverseProjection(2896, 2896, camera_parameters1);
[~, ~, ~, x0, y0] = reverseProjection(2896, 2896, camera_parameters0);

xi1 = camera_parameters1.xi;

xp1 = xs1 ./ (xi1 + zs1);
yp1 = ys1 ./ (xi1 + zs1);

xi0 = camera_parameters0.xi;

xp0 = xs0 ./ (xi0 + zs0);
yp0 = ys0 ./ (xi0 + zs0);

resampled_p1 = interp2(x1, y1, estimated_p1, xp1, yp1, 'makima', 0);
resampled_q1 = interp2(x1, y1, estimated_q1, xp1, yp1, 'makima', 0);
resampled_p0 = interp2(x0, y0, estimated_p0, xp0, yp0, 'makima', 0);
resampled_q0 = interp2(x0, y0, estimated_q0, xp0, yp0, 'makima', 0);

fused_p = [ resampled_p1 ; (resampled_p1(end, :)-resampled_p0(1, :))/2 ; -resampled_p0 ];
fused_q = [ resampled_q1 ; (resampled_q1(end, :)-resampled_q0(1, :))/2 ; -resampled_q0 ];

%% Rho estimation
theta = incr : incr : pi - incr;
phi = 0 : incr : 2*pi - incr;

ln_rho = rangeMapOmni(fused_p, fused_q, lambda);
rho = exp(ln_rho);

[p, t] = meshgrid(phi, theta);
X = rho.*sin(t).*cos(p);
Y = rho.*sin(t).*sin(p);
Z = rho.*cos(t);

pcshow([X(:), Y(:), Z(:)]);
